// GetFileClient.java

import java.net.*;
import java.io.*;

public class GetFileClient {

    public static void main(String[] args) throws IOException {
        /*
         * Accepted arguments: server address and port
         *
         */
        InetAddress addr = null;
        int port = -1;

        /* check arguments */
        try {
            if (args.length == 2) {
                addr = InetAddress.getByName(args[0]);
                port = Integer.parseInt(args[1]);
            } else {
                System.out.println("Usage: java GetFileClient serverAddr serverPort");
                System.exit(1);
            }
        } //try
        // Try to manage differently each possible exception
        catch (Exception e) {
            System.out.println("Error: ");
            e.printStackTrace();
            System.out.println("Usage: java GetFileClient serverAddr serverPort");
            System.exit(2);
        }

        // objects needed by the client to access local file
        Socket socket = null;
        DataInputStream inSock = null;
        DataOutputStream outSock = null;

        // keyboard input stream
        BufferedReader stdIn = new BufferedReader(new InputStreamReader(System.in));
        String dirName = null;
        System.out.print("\n^D(Unix)/^Z(Win)+enter to quit, else input the name of the directory to get: ");

        try {
            if ((dirName = stdIn.readLine()) != null) {

                // socket and input/output streams creation
                try {
                    socket = new Socket(addr, port);
                    // set timeout to avoid infinite blocks
                    socket.setSoTimeout(30000);
                    System.out.println("Socket created: " + socket);
                    inSock = new DataInputStream(socket.getInputStream());
                    outSock = new DataOutputStream(socket.getOutputStream());
                } catch (IOException ioe) {
                    System.out.println("Error while creating streams: ");
                    ioe.printStackTrace();
                    System.exit(1);
                } catch (Exception e) {
                    System.out.println("Error while creating socket: ");
                    e.printStackTrace();
                    System.exit(2);
                }

                // send name and implementation of the communication protocol
                try {
                    // send directory name
                    outSock.writeUTF(dirName);
                    System.out.println("Directory name sent: " + dirName);
                    String receivedFilename;
                    long bytesize;
                    File currentFile;
                    FileOutputStream outCurrentFile;
                    // read the file name. If it is equal to "?", it means that
                    // the server could not find the requested directory
                    while ((receivedFilename = inSock.readUTF()) != null) {
                        if (!receivedFilename.equals("?")) {
                            // read file size
                            bytesize = inSock.readLong();
                            // check if the file exists. If it does, delete it. If it does
                            // not, create it
                            currentFile = new File(receivedFilename);
                            if (currentFile.exists()) {
                                currentFile.delete();
                            }
                            outCurrentFile = new FileOutputStream(receivedFilename);
                            System.out.println("Transferring " + receivedFilename + ", size: " + bytesize + " bytes");
                            // file transfer
                            FileUtility.n_byte_file_transfer(inSock, new DataOutputStream(outCurrentFile), bytesize);
                            // close file
                            outCurrentFile.close();
                        } else {
                            System.err.println("Error: " + dirName + " does not exist");
                            socket.close();
                            System.exit(3);
                        }
                    } //while
                }
                /*
                 * NOTE: upon reaching EOF, readUTF throws an exception. We catch
                 * it here, close the socket and terminate successfully the client
                 */ catch (EOFException e) {
                    System.out.println("End of transmission, terminating...");
                    // close socket
                    socket.close();
                    System.out.println("GetFileClient: terminating...");
                    System.exit(0);
                } // different exceptions can be catched here
                catch (Exception e) {
                    System.out.println("Error while sendinf " + dirName + ": ");
                    e.printStackTrace();
                    socket.close();
                    // exit with error
                    System.exit(3);
                }
            }// if
        }  // catch all exception not caught in the while loop
        // for example unexpected connection termination
        catch (Exception e) {
            System.err.println("Unexpected exception: ");
            e.printStackTrace();
            System.err.println("Terminating!");
            System.exit(4);
        }
    } // main
} // GetFileClient

