// Concurrent GetFileServer

import java.io.*;
import java.net.*;

// Thread started for each accepted request
class GetFileServerThread extends Thread {

    private Socket clientSocket = null;

    // costruttore
    public GetFileServerThread(Socket clientSocket) {
        this.clientSocket = clientSocket;
    }

    public void run() {
        DataInputStream inSock;
        DataOutputStream outSock;

        System.out.println("Child process: " + Thread.currentThread().getId());
        try {
            try {
                // input and output streams creation
                inSock = new DataInputStream(clientSocket.getInputStream());
                outSock = new DataOutputStream(clientSocket.getOutputStream());
            } catch (IOException ioe) {
                System.out.println("Error while creating input/output stream: ");
                ioe.printStackTrace();
                return;
            } catch (Exception e) {
                System.out.println("Error while creating input/output stream: ");
                e.printStackTrace();
                return;
            }

            /* send file */
            String dirName = null;
            try {
                // ricezione del nome
                dirName = inSock.readUTF();
                File currentDir = new File(dirName);
                if (currentDir.exists() && currentDir.isDirectory()) {
                    File[] files = currentDir.listFiles();
                    File fileCorr;
                    for (int i = 0; i < files.length; i++) {
                        fileCorr = files[i];
                        System.out.println("File: " + fileCorr.getName());
                        if (fileCorr.isFile()) {
                            // send file name
                            outSock.writeUTF(fileCorr.getName());
                            // send length
                            outSock.writeLong(fileCorr.length());
                            // transfer data
                            FileUtility.n_byte_file_transfer(new DataInputStream(new FileInputStream(fileCorr.getAbsolutePath())), outSock, fileCorr.length());
                        }
                    }
                } else {
                    // on error, send a special string
                    outSock.writeUTF("?");
                }
                // close the connection
                clientSocket.close();
            } catch (SocketTimeoutException ste) {
                System.out.println("Timeout: ");
                ste.printStackTrace();
                clientSocket.close();
            } catch (Exception e) {
                System.out.println("Exception: ");
                e.printStackTrace();
                System.out.println("Terminating...");
                clientSocket.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
            // unexpected exception
            System.out.println("Unexpected exception, GetFileServerThread: terminating...");
        }
        System.out.println("Child process terminating: " + Thread.currentThread().getId());
    } // run
} // GetFileServerCon

public class GetFileServerCon {

    /**
     * Mandatory: listening port
     */
    public static void main(String[] args) throws IOException {

        // Server listening port
        int port = -1;

        /* check arguments */
        try {
            if (args.length == 1) {
                port = Integer.parseInt(args[0]);
                // check that port is in the unprivileged range 1024-65535
                if (port < 1024 || port > 65535) {
                    System.out.println("Usage: java LineServer [serverPort>1024]");
                    System.exit(1);
                }
            } else {
                System.out.println("Usage: java GetFileServerCon port");
                System.exit(1);
            }
        } // try
        catch (Exception e) {
            System.out.println("Error: ");
            e.printStackTrace();
            System.out.println("Usage: java GetFileServerCon port");
            System.exit(1);
        }

        ServerSocket serverSocket = null;
        Socket clientSocket = null;

        try {
            serverSocket = new ServerSocket(port);
            serverSocket.setReuseAddress(true);
            System.out.println("GetFileServerCon: started");
            System.out.println("Server: socket created: " + serverSocket);
        } catch (Exception e) {
            System.err.println("Server: error while creating socket: " + e.getMessage());
            e.printStackTrace();
            System.exit(1);
        }

        try {

            while (true) {
                System.out.println("Server: waiting for request\n");

                try {
                    // waits for a connection request until the timout trigger
                    clientSocket = serverSocket.accept();
                    // set timeout
                    clientSocket.setSoTimeout(60000);
                    System.out.println("Server: accepted connection: " + clientSocket);
                } catch (Exception e) {
                    System.err.println("Server: error while accepting connection: " + e.getMessage());
                    e.printStackTrace();
                    // server endless loop
                    continue;
                }

                // service managed by a new thread
                try {
                    new GetFileServerThread(clientSocket).start();
                } catch (Exception e) {
                    System.err.println("Server: error in child thread: " + e.getMessage());
                    e.printStackTrace();
                    // server endless loop
                    continue;
                }

            } // while
        } // here we catch unexpected exceptions
        catch (Exception e) {
            e.printStackTrace();
            System.out.println("GetFileServerCon: terminating...");
            System.exit(2);
        }

    }
} // GetFileServerCon

